from typing import List, Dict, Optional

from .base_ssdb import BaseSsdb
from .utils import SsdbResponseUtils


class BaseSsdbKV(BaseSsdb):

    def set(self, key: any, value: any, ttl: int = -1):
        pass

    def exists(self, key: any) -> bool:
        pass

    def get(self, key: any) -> Optional[bytes]:
        pass

    def ttl(self, key: any) -> int:
        pass

    def incr(self, key: any, num: int) -> int:
        pass

    def remove(self, key: any):
        pass

    def multi_set(self, kv_dict: Dict[any, any]):
        pass

    def multi_get(self, keys: List[any]) -> Dict[bytes, bytes]:
        pass

    def multi_del(self, keys: List[any]):
        pass


class SsdbKV(BaseSsdbKV):

    def set(self, key: any, value: any, ttl: int = -1):
        if ttl is None or ttl <= 0:
            self.execute_command('set', key, value)
        else:
            self.execute_command('setx', key, value, ttl)

    def exists(self, key: any) -> bool:
        result = self.execute_command('exists', key)
        return True if int(result) == 1 else False

    def get(self, key: any) -> Optional[bytes]:
        return self.execute_command('get', key)

    def ttl(self, key: any) -> int:
        return int(self.execute_command('ttl', key))

    def incr(self, key: any, num: int) -> int:
        result = self.execute_command('incr', key, num)
        return int(result)

    def remove(self, key: any):
        self.execute_command('del', key)

    def multi_set(self, kv_dict: Dict[any, any]):
        pairs = SsdbResponseUtils.encode_dict_to_pairs(kv_dict)
        self.execute_command('multi_set', *pairs)

    def multi_get(self, keys: List[any]) -> Dict[bytes, bytes]:
        result = self.execute_command('multi_get', *keys)

        return SsdbResponseUtils.response_to_map(result)

    def multi_del(self, keys: List[any]):
        self.execute_command('multi_del', *keys)



import json
from typing import Dict, Union

import pyssdb

from .ssdb_hash import SsdbHash
from .ssdb_kv import SsdbKV
from .ssdb_sorted_set import SsdbSortedSet
from .utils import SsdbResponseUtils


#
# @author anhlt
#
class SsdbClient(SsdbKV, SsdbHash, SsdbSortedSet):

    def __init__(self, host: str, port: int):
        # TODO: Replace this client to fixed connection pool properly
        self.ssdb: pyssdb.Client = pyssdb.Client(host, port)

        info = self.info()
        print('SSDB info:')
        print(json.dumps(info, ensure_ascii=False, indent=2))

    def auth(self, password: str) -> bool:
        result = self.ssdb.execute_command('auth', password)
        return True if int(result) == 1 else False

    def info(self) -> Dict[str, str]:
        result_dict = self.ssdb.execute_command('info')
        return SsdbResponseUtils.to_str_map(result_dict)

    def dbsize(self) -> int:
        result_dict = self.ssdb.execute_command('dbsize')
        return int(result_dict)

    def execute_command(self, cmd: str, *args) -> Union[None, list, int, bool]:
        return self.ssdb.execute_command(cmd, *args)



import logging
import traceback
from typing import Dict, List, Tuple


def get_msg(e: Exception) -> str:
    return "".join(
        traceback.TracebackException.from_exception(e.__cause__).format()) if e.__cause__ is not None else str(
        e)


class SsdbResponseUtils:

    def __init__(self):
        raise Exception("Not allow to create a new intance of this class")

    @classmethod
    def encode_dict_to_pairs(cls, dict: Dict[any, any]) -> List[any]:
        pairs = []
        for k, v in dict.items():
            pairs.append(k)
            pairs.append(v)
        return pairs

    @classmethod
    def response_to_map(cls, response) -> Dict[bytes, bytes]:
        result_dict = {}
        for k, v in zip(response[::2], response[1::2]):
            result_dict[k] = v
        return result_dict

    @classmethod
    def response_to_str_map(cls, response) -> Dict[str, str]:
        result_dict = {}
        for k, v in zip(response[::2], response[1::2]):
            try:
                id = k.decode("utf-8")
                value = v.decode('utf-8')
                result_dict[id] = value
            except Exception as e:
                logging.error(f"response_to_str_map: {get_msg(e)}")

        return result_dict

    @classmethod
    def response_to_pair_list(cls, response) -> List[Tuple[bytes, bytes]]:
        return [
            (k, v)
            for k, v in zip(response[::2], response[1::2])
        ]

    @classmethod
    def to_str_map(cls, dict: Dict[bytes, bytes]) -> Dict[str, str]:
        result_dict = {}
        for k, v in dict.items():
            try:
                id = str(k.decode("utf-8"))
                value = str(v.decode('utf-8'))
                result_dict[id] = value
            except Exception as e:
                logging.error(f"to_str_map: {get_msg(e)}")

        return result_dict

    @classmethod
    def to_zset_score_dict(cls, response) -> Dict[str, float]:
        result_dict = {}
        for i, (k, v) in enumerate(zip(response[::2], response[1::2])):
            try:
                id = k.decode("utf-8")
                score = float(v.decode('utf-8'))
                result_dict[id] = score
            except Exception as e:
                logging.error(f"to_zset_score_dict: {get_msg(e)}")

        return result_dict

    @classmethod
    def to_rank_to_id_map(cls, response, ranks: List[int], start_rank: int) -> Dict[int, str]:
        result_dict = {}

        set_ranks = set(ranks)
        for i, (k, v) in enumerate(zip(response[::2], response[1::2])):
            try:
                id = k.decode("utf-8")
                score = float(v.decode('utf-8'))
                rank = start_rank + i
                if rank in set_ranks:
                    result_dict[start_rank + i] = id
            except Exception as e:
                logging.error(f"to_map: {get_msg(e)}")

        return result_dict



from py_ssdb_client import SsdbClient


def run_test_simple_kv(client: SsdbClient):
    print('=========KEY-VALUE TESTING===========')
    key = 'simple_key'
    client.set(key, 'this is a sentence')
    result = client.get(key)
    print(f'Get: {result}')

    result = client.incr('k', 10)
    print(f'incr: {result}')

    client.remove(key)
    result = client.get(key)
    print(f'get after remove: {result}')
    assert result is None

    result = client.exists('k')
    print(f'exists: {result}')
    assert result == True

    result = client.exists(key)
    print(f'exists: {result}')
    assert result == False

    result = client.multi_set({'a': 1, 'b': 2})
    print(f'multi_set: {result}')
    result = client.multi_get(['a', 'b'])
    print(f'multi_get: {result}')


def run_test_hash(client: SsdbClient):
    print('=========HASH TESTING===========')
    name = "hash"

    client.multi_hset(name, {'a': 34, 'b': 14, 'c': 100})
    client.hset(name, 's', 33)

    result = client.multi_hget(name, ['a', 'c', 's'])
    print(f'multi_hget: {result}')
    result = client.multi_hget('hash15', ['a', 'c'])
    print(f'multi_hget: {result}')

    result = client.hdel(name, 's')
    result = client.hget(name, 's')
    print(f'hget: {int(result) if result is not None else ""}')

    result = client.hsize(name)
    print(f'hsize: {int(result) if result is not None else ""}')

    result = client.hsize('hash15')
    print(f'hsize: {int(result) if result is not None else ""}')

    result = client.hexists('hash15', 'a')
    print(f'hexists: {result if result is not None else ""}')
    assert result == False

    print('Test HIncr')
    client.hset('hash_incr', 'a', 1)
    client.hincr('hash_incr', 'a', 100)
    result = client.hget('hash_incr', 'a')
    print(f'hash_incr: {int(result) if result is not None else ""}')

    result = client.hkeys(name,'a', 'z', 10)
    print(f'hkeys: {result if result is not None else ""}')

    result = client.hgetall(name)
    print(f'hgetall: {result if result is not None else ""}')


def run_test_sorted_set(client: SsdbClient):
    print('=========SORTED SET TESTING===========')
    client.multi_zset('sorted_set', {'a': 34, 'b': 14, 'c': 100})
    result = client.multi_zget('sorted_set', ['a', 'c', 's'])
    print(f'sorted_set: {result}')

    result = client.zrange('sorted_set', 0, 2)
    print(f'zrange: {result}')
    result = client.zrrange('sorted_set', 0, 2)
    print(f'zrrange: {result}')


if __name__ == "__main__":
    client = SsdbClient(
        host="127.0.0.1",
        port=8888
    )

    client.auth('sadsdgv')
    run_test_simple_kv(client)
    run_test_hash(client)
    run_test_sorted_set(client)



from typing import List, Dict, Tuple

from .base_ssdb import BaseSsdb
from .utils import SsdbResponseUtils


class BaseSsdbSortedSet(BaseSsdb):

    def zset(self, name: str, key: str, score: float):
        pass

    def multi_zset(self, name: str, score_mapping: Dict[str, float]):
        pass

    def multi_zget(self, name: str, keys: List[str]) -> Dict[str, float]:
        pass

    def zrank(self, name: str, key: str) -> int:
        pass

    def zrrank(self, name: str, key: str) -> int:
        pass

    def zrange(self, name: str, offset: int, size: int) -> List[Tuple[str, float]]:
        pass

    def zrrange(self, name: str, offset: int, size: int) -> List[Tuple[str, float]]:
        pass

    def multi_get_item_by_ranking(self, name: str, ranks: List[int]) -> Dict[int, str]:
        pass

    def zdel(self, name: str, key: str):
        pass

    def multi_zdel(self, name: str, keys: List[str]) -> bool:
        pass

    def zclear(self, name: str):
        pass


class SsdbSortedSet(BaseSsdbSortedSet):

    def zset(self, name: str, key: str, score: float):
        result = self.execute_command(
            'zset',
            name,
            key,
            score
        )

    def multi_zset(self, name: str, score_mapping: Dict[str, float]):
        pairs = SsdbResponseUtils.encode_dict_to_pairs(score_mapping)
        self.execute_command('multi_zset', name, *pairs)

    def multi_zget(self, name: str, keys: List[str]) -> Dict[str, float]:
        result = self.execute_command('multi_zget', name, *keys)

        return {
            k.decode('utf-8'): float(v)
            for k, v in SsdbResponseUtils.response_to_map(result).items()
        }

    def zrank(self, name: str, key: str) -> int:
        result = self.execute_command('zrank', name, key)
        return int(result) if result is not None else None

    def zrrank(self, name: str, key: str) -> int:
        result = self.execute_command('zrrank', name, key)
        return int(result) if result is not None else None

    def zrange(self, name: str, offset: int, size: int) -> List[Tuple[str, float]]:
        result = self.execute_command('zrange', name, offset, size)
        return [
            (k.decode('utf-8'), float(v))
            for k, v in SsdbResponseUtils.response_to_pair_list(result)
        ]

    def zrrange(self, name: str, offset: int, size: int) -> List[Tuple[str, float]]:
        result = self.execute_command('zrrange', name, offset, size)
        return [
            (k.decode('utf-8'), float(v))
            for k, v in SsdbResponseUtils.response_to_pair_list(result)
        ]

    def multi_get_item_by_ranking(self, name: str, ranks: List[int]) -> Dict[int, str]:
        actual_result_map: Dict[int, str] = {}

        if len(ranks) > 0:
            start_rank, end_rank = max(0, min(ranks)), max(ranks)
            response = self.execute_command('zrange', name, start_rank, end_rank + 1)
            rank_to_id_map = SsdbResponseUtils.to_rank_to_id_map(response, ranks, start_rank)
            for rank in ranks:
                if rank in rank_to_id_map.keys():
                    actual_result_map[rank] = rank_to_id_map[rank]

        return actual_result_map

    def zdel(self, name: str, key: str):
        result = self.execute_command('zdel', name, key)

    def multi_zdel(self, name: str, keys: List[str]) -> bool:
        self.execute_command('multi_zdel', name, *keys)
        return True

    def zclear(self, name: str):
        result = self.execute_command('zclear', name)



from typing import List, Dict, Optional

from .base_ssdb import BaseSsdb
from .utils import SsdbResponseUtils


class BaseSsdbHash(BaseSsdb):

    def hincr(self, name: str, k: any, num: int) -> int:
        """
        Increase the given key value by num

        :param name: any
        :param k: any
        :param num
        :return:
        """
        pass

    def hset(self, name: str, k: any, v: any):
        pass

    def hget(self, name: str, k: any) -> Optional[bytes]:
        pass

    def hsize(self, name: str) -> int:
        pass

    def hexists(self, name: str, k: any) -> bool:
        pass

    def hdel(self, name: str, k: any):
        pass

    def hclear(self, name: str):
        pass

    def multi_hset(self, name: str, data: Dict[any, any]):
        pass

    def multi_hget(self, name: str, keys: List[str]) -> Dict[bytes, bytes]:
        pass

    def hkeys(self, name: str, key_start: any, key_end: any, limit: int) -> List[bytes]:
        pass

    def hlist(self, name_start: str, name_end: str, limit: int) -> List[str]:
        pass

    def hrlist(self, name_start: str, name_end: str, limit: int) -> List[str]:
        pass

    def hgetall(self, name: str) -> Dict[bytes, bytes]:
        pass


class SsdbHash(BaseSsdbHash):

    def hincr(self, name: str, k: any, num: int) -> int:
        result = self.execute_command('hincr', name, k, num)
        return int(result)

    def hset(self, name: str, k: any, v: any):
        self.execute_command('hset', name, k, v)

    def hget(self, name: str, k: any) -> Optional[bytes]:
        return self.execute_command('hget', name, k)

    def hsize(self, name: str) -> int:
        return int(self.execute_command('hsize', name))

    def hexists(self, name: str, k: any) -> bool:
        result = self.execute_command('hexists', name, k)
        return True if int(result) == 1 else False

    def hdel(self, name: str, k: any):
        self.execute_command('hdel', name, k)

    def hclear(self, name: str):
        self.execute_command('hclear', name)

    def multi_hset(self, name: str, data: Dict[any, any]):
        pairs = SsdbResponseUtils.encode_dict_to_pairs(data)
        self.execute_command('multi_hset', name, *pairs)

    def multi_hget(self, name: str, keys: List[str]) -> Dict[bytes, bytes]:
        result = self.execute_command('multi_hget', name, *keys)

        return SsdbResponseUtils.response_to_map(result)

    def hkeys(self, name: str, key_start: any, key_end: any, limit: int) -> List[bytes]:
        result = self.execute_command('hkeys', name, key_start, key_end, limit)
        return list(result)

    def hlist(self, name_start: str, name_end: str, limit: int) -> List[str]:
        result = self.execute_command('hlist', name_start, name_end, limit)
        return list(result)

    def hrlist(self, name_start: str, name_end: str, limit: int) -> List[str]:
        result = self.execute_command('hrlist', name_start, name_end, limit)
        return list(result)

    def hgetall(self, name: str) -> Dict[bytes, bytes]:
        result = self.execute_command('hgetall', name)
        return SsdbResponseUtils.response_to_map(result)



import setuptools
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="py-ssdb-client",
    version="0.0.8",
    author="Andy Le",
    author_email="tauit.dnmd@gmail.com",
    description="A SSDB client for Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/andy1xx8/py-ssdb-client",
    project_urls={
        "Bug Tracker": "https://github.com/andy1xx8/py-ssdb-client/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(where="src"),
    package_dir={"": "src"},
    include_package_data=True,
    python_requires=">=3.6",
    install_requires=["pyssdb==0.4.2", "py-profiler==0.2.4"]
)



from typing import Dict, Union


class BaseSsdb:

    def auth(self, password: str) -> bool:
        """
        Auth this ssdb with the given password

        :param password: the given password
        :return: True if auth successfully or there is no password required
        """
        pass

    def info(self) -> Dict[str, str]:
        """
        Get the detail information about this ssdb server.

        :return: Dict[str, str]
        """
        pass

    def dbsize(self) -> int:
        """
        Get the approximate size of this ssdb server

        :return: size in bytes
        """
        pass

    def execute_command(self, cmd: str, *args) -> Union[None, list, int, bool]:
        pass
